# -*- coding: utf-8 -*-

from typing import Union, Optional
import datetime

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError, to_datetime

FILE_TIMES = bidict(
    access=QtCore.QFileDevice.FileAccessTime,
    birth=QtCore.QFileDevice.FileBirthTime,
    metadata_change=QtCore.QFileDevice.FileMetadataChangeTime,
    modification=QtCore.QFileDevice.FileModificationTime,
)

QtCore.QFileDevice.__bases__ = (core.IODevice,)


class FileDevice(QtCore.QFileDevice):
    def set_file_time(
        self, file_time: Union[QtCore.QDateTime, datetime.datetime], typ: str
    ) -> bool:
        """Set file time.

        Allowed values are "access", "birth", "metadata_change", "modification"

        Args:
            typ: file time to use

        Raises:
            InvalidParamError: file time does not exist
        """
        if typ not in FILE_TIMES:
            raise InvalidParamError(typ, FILE_TIMES)
        return self.setFileTime(file_time, FILE_TIMES[typ])

    def get_file_time(self, typ: str) -> Optional[datetime.datetime]:
        """Return current file time.

        Possible values: "access", "birth", "metadata_change", "modification"

        Returns:
            file time
        """
        if typ not in FILE_TIMES:
            raise InvalidParamError(typ, FILE_TIMES)
        date = self.fileTime(FILE_TIMES[typ])
        if not date:
            return None
        return to_datetime(date)



# -*- coding: utf-8 -*-

from qtpy import QtCore


class Margins(QtCore.QMargins):
    def __repr__(self):
        return f"Margins({self.left()}, {self.top()}, {self.right()}, {self.bottom()})"

    def __bool__(self):
        return not self.isNull()

    def __iter__(self):
        yield self.left()
        yield self.top()
        yield self.right()
        yield self.bottom()


if __name__ == "__main__":
    margins = Margins(0, 0, 0, 0)
    print(bool(margins))



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QFile.__bases__ = (core.FileDevice,)


class File(QtCore.QFile):
    def __repr__(self):
        return f"File('{self.fileName()}')"

    def __str__(self):
        return self.fileName()



# -*- coding: utf-8 -*-

"""Webenginewidgets module.

contains QtWebEngineWidgets-based classes
"""

import logging

logger = logging.getLogger(__name__)

try:
    from .webenginehistoryitem import WebEngineHistoryItem
    from .webenginehistory import WebEngineHistory
    from .webengineview import WebEngineView
    from .webenginepage import WebEnginePage
    from .webengineprofile import WebEngineProfile
    from .webenginedownloaditem import WebEngineDownloadItem
    from .webenginescript import WebEngineScript

    __all__ = [
        "WebEngineHistoryItem",
        "WebEngineHistory",
        "WebEngineView",
        "WebEnginePage",
        "WebEngineProfile",
        "WebEngineDownloadItem",
        "WebEngineScript",
    ]
except ModuleNotFoundError:
    logger.warning("Tried to import WebEngine module but not installed.")



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtCore

from prettyqt import core

QtCore.QAnimationGroup.__bases__ = (core.AbstractAnimation,)


class AnimationGroup(QtCore.QAnimationGroup):
    def __getitem__(self, index: Union[int, slice]):
        if isinstance(index, int):
            if index < 0:
                index = len(self) + index
            return self.animationAt(index)
        else:
            anims = [self.animationAt(i) for i in range(len(self))]
            return anims[index]

    def __setitem__(self, index: int, value: QtCore.QAbstractAnimation):
        old = self.animationAt(index)
        self.removeAnimation(old)
        self.insertAnimation(index, value)

    def __len__(self):
        return self.animationCount()

    def __delitem__(self, index: int):
        animation = self[index]
        self.removeAnimation(animation)

    def __add__(self, other: QtCore.QAbstractAnimation):
        self.addAnimation(other)
        return self

    def add_property_animation(
        self, obj: QtCore.QObject, attribute: str
    ) -> core.PropertyAnimation:
        anim = core.PropertyAnimation(obj, attribute.encode())
        self.addAnimation(anim)
        return anim



# -*- coding: utf-8 -*-

try:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
except ImportError:
    from PySide2 import QtWebEngineWidgets

from prettyqt import core


class WebEngineHistoryItem(QtWebEngineWidgets.QWebEngineHistoryItem):
    def get_url(self) -> core.Url:
        return core.Url(self.url())

    def get_last_visited(self) -> core.DateTime:
        return core.DateTime(self.lastVisited())


if __name__ == "__main__":
    item = WebEngineHistoryItem()
